<?php
/**
  * Class represents upload action.
  *
  * @author Marek Blotny <marek@mbsoftware.pl>
  * @copyright Copyright &copy; 2006, Marek Blotny <{@link http://filebrowser.mbsoftware.pl http://filebrowser.mbsoftware.pl}>
  * @license http://www.opensource.org/licenses/mit-license.php The MIT License
  *
  * @package onlinefilebrowser.action
  */


/**
  * Class represents upload action, it moves uploaded file into final destination. Class also performs validation, checks if file name is valid
  * based on Browser :: isFileNameValid() method. 
  *
  * If uploaded file is a graphical file and property in configuration file <i>browser.thumbnail.create</i> is set to true then creates thumbnail.
  * 
  * @author Marek Blotny <marek@mbsoftware.pl>
  * @copyright Copyright &copy; 2006, Marek Blotny <{@link http://filebrowser.mbsoftware.pl http://filebrowser.mbsoftware.pl}>
  * @license http://www.opensource.org/licenses/mit-license.php The MIT License
  *
  * @package onlinefilebrowser.action
  */
class UploadFormAction extends ActionTemplate {
	
	/**
	  * Process upload file, checks if file is valid, and saves file in final destination. 
	  * If it's a graphical file then thumbnail can be created (depends on configuration).
	  *
	  * @access public
	  */
	function perform() {
		
		$templateData = array();
	
		$smarty =& $this->getSmarty();
		$smarty->assign('result', 'failed');
		
		if(isset($_FILES['file']['tmp_name']) && is_uploaded_file($_FILES['file']['tmp_name'])) {
						
			// base + relative
			$relativePath = Browser :: getRelativePath($_REQUEST['relativePath']);
			$directory = realpath(Browser :: getBrowserRoot() . $relativePath);

					
			// base + relative + name
			$uploadfile = 	$directory.
					Browser_Utilities :: getSeparator().
					$_FILES['file']['name'];
					
			Browser_Utilities :: log("[UploadFormAction.perform] file uploaded: ". $uploadfile, "info");
			
			// validate file
			if (Browser :: isFileNameValid($_FILES['file']['name'])) {
				
				if (!move_uploaded_file($_FILES['file']['tmp_name'], $uploadfile)) {
					Browser_Utilities :: log("[UploadFormAction.perform] error while file move_uploaded_file: ". $_FILES['file']['name'], "error");
					$smarty->assign('result', 'failed');
				} else {
					// success !!
					$smarty->assign('result', 'success');
					
					// create thumbnail for a file
					// if it's a graphical file and if in configuration this option is enabled
					if (BrowserHelper :: isGraphicalFile($_FILES['file']['name']) && Browser_Utilities :: getValueFromConfiguration("browser.thumbnail.create") == "true") {
						UploadFormAction :: createThumbnail($uploadfile);
					}
				}
			} else {
				$smarty->assign('result', 'failed');
				Browser_Utilities :: log("[UploadFormAction.perform] file name is not allowed: ". $_FILES['file']['name'], "warn");
			
			}
		}
		
		$smarty->assign('labels', Browser_Utilities :: loadProperites("conf". Browser_Utilities :: getSeparator() .Browser_Utilities :: getValueFromConfiguration("resource.labels.file")));
		$smarty->display("uploadForm.tpl");
	}
	
	/**
	  * Method creates thumbnail.
	  *
	  * @access private
	  *
	  * @param uploadfile is a full path to uploaded file
	  */
	function createThumbnail($uploadfile) {
		
		$directory = dirname($uploadfile);
		
		// create thumbnail in memory
		$image = ImagesUtils :: createThumbnail($uploadfile);
		
		// create directory .thumbnail if not exist
		$thumbnailDirectory = 	$directory.
					Browser_Utilities :: getSeparator().Browser_Utilities :: getValueFromConfiguration("browser.thumbnail.directory");
		
		if ( !is_dir( $thumbnailDirectory ) ) { mkdir($thumbnailDirectory); }
		
		$thumbnailFilename = 	$thumbnailDirectory.
					Browser_Utilities :: getSeparator().basename($uploadfile);
		
		$fileExt =  BrowserHelper :: getFileExtension(basename($uploadfile));
		if ( $fileExt == 'jpeg' || $fileExt == 'jpg') {
			imagejpeg($image, $thumbnailFilename);
			Browser_Utilities :: log("[UploadFormAction.perform] thumbnail created for file: ". basename($uploadfile), "info");
		} else if ($fileExt == 'gif') {
			imagegif($image, $thumbnailFilename);
			Browser_Utilities :: log("[UploadFormAction.perform] thumbnail created for file: ". basename($uploadfile), "info");
		} else if ($fileExt == 'png') {
			imagepng($image, $thumbnailFilename);
			Browser_Utilities :: log("[UploadFormAction.perform] thumbnail created for file: ". basename($uploadfile), "info");
		}
		
		imagedestroy($image);
	}
}

?>
